<?php
/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\ExtViSualMerchandiser\Model\Category;

use Magento\Catalog\Model\Category\Product\PositionResolver;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\DB\Select;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Module\Manager;
use Magento\Store\Api\StoreManagementInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\VisualMerchandiser\Model\Position\Cache;
use Magento\VisualMerchandiser\Model\Sorting;

/**
 * Class Products
 * @package FiloBlu\ExtViSualMerchandiser\Model\Category
 */
class Products extends \Magento\VisualMerchandiser\Model\Category\Products
{
    /** @var ScopeConfigInterface */
    protected $scopeConfig;

    /** @var StoreManagerInterface  */
    protected $storeManager;

    /**
     * Products constructor.
     * @param ProductFactory $productFactory
     * @param Manager $moduleManager
     * @param Cache $cache
     * @param Sorting $sorting
     * @param ScopeConfigInterface $scopeConfig
     * @param PositionResolver|null $positionResolver
     */
    public function __construct(
        ProductFactory $productFactory,
        Manager $moduleManager,
        Cache $cache,
        Sorting $sorting,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        PositionResolver $positionResolver = null
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
        //TODO: TEMP FIND A BETTER WAY TO FIX MAGENTO 2.4 ISSUE
        if(class_exists(\Magento\VisualMerchandiser\Model\Resolver\QuantityAndStock::class)){
            $quantityStock = ObjectManager::getInstance()->get(\Magento\VisualMerchandiser\Model\Resolver\QuantityAndStock::class);
            parent::__construct($productFactory, $moduleManager, $cache, $sorting, $quantityStock, $positionResolver);
            return;
        }

        parent::__construct($productFactory, $moduleManager, $cache, $sorting, $positionResolver);
    }

    /**
     * @param int $categoryId
     * @param int $store
     * @return Collection
     * @throws LocalizedException
     */
    public function getCollectionForGrid($categoryId, $store = null, $productPositions = null)
    {
        $websiteId = null;
        $storeId = null;

        if($store === null)
        {
           $default = $this->storeManager->getDefaultStoreView();
            $websiteId = $default->getWebsiteId();
            $storeId = $default->getId();
        }else{
            $storeId = $store;
            $websiteId = $this->storeManager->getStore($store)->getWebsiteId();
        }


        /** @var Collection $collection */
        $collection = $this->getFactory()->create()
            ->getCollection()
            ->addAttributeToSelect([
                'sku',
                'name',
                'status',
                'price',
                'small_image',
                'image',
                'thumbnail'
            ]);

        if ($store !== null) {
            $collection->addStoreFilter($storeId);
        }

        $collection->getSelect()
            ->where('at_position.category_id = ?', $categoryId);

        if ($this->_moduleManager->isEnabled('Magento_CatalogInventory')) {
            if ($this->getStockId() == Stock::DEFAULT_STOCK_ID) {
                $collection->joinField(
                    'stock',
                    'cataloginventory_stock_item',
                    'qty',
                    'product_id=entity_id',
                    ['stock_id' => $this->getStockId()],
                    'left'
                );
            } else {
                $collection->joinField(
                    'stock',
                    'inventory_stock_' . $this->getStockId(),
                    'quantity',
                    'sku=sku',
                    null,
                    'left'
                );
            }
        }

        $cache = $this->_cache->getPositions($this->_cacheKey);

        if ($cache === false) {
            $collection->joinField(
                'position',
                'catalog_category_product',
                'position',
                'product_id=entity_id',
                null,
                'left'
            );
            $collection->setOrder('position', $collection::SORT_ORDER_ASC);

            // Cache the positions initially
            $_collection = clone $collection;

            $positions = [];
            $idx = 0;
            foreach ($_collection as $item) {
                $positions[$item->getId()] = $idx;
                $idx++;
            }

            $this->_cache->saveData($this->_cacheKey, $positions);
        } else {
            $collection->getSelect()
                ->reset(Select::WHERE)
                ->reset(Select::HAVING);

            $collection->addAttributeToFilter('entity_id', ['in' => array_keys($cache)]);
        }

        return $collection;
    }

    /**
     * @return int
     */
    protected function getStockId()
    {
        $stock = $this->scopeConfig->getValue('visualmerchandiser/options/default_stock', ScopeInterface::SCOPE_STORE);
        if ($stock) {
            return $stock;
        }
        return Stock::DEFAULT_STOCK_ID;
    }
}
