<?php

namespace FiloBlu\ExtViSualMerchandiser\Ui\Component\Listing\Column;

use Magento\Catalog\Api\ProductAttributeRepositoryInterface as AttributeRepository;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Eav\Model\ResourceModel\Entity\Attribute;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;

/**
 * Class Color
 * @package FiloBlu\ExtViSualMerchandiser\Ui\Component\Listing\Column
 */
class Color extends Column
{
    /**
     * @var Attribute
     */
    protected $_eavEntityAttribute;
    /**
     * @var AttributeRepository
     */
    protected $attributeRepository;
    /**
     * @var Collection
     */
    protected $collection;

    /**
     * Color constructor.
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param AttributeRepository $attributeRepository
     * @param Attribute $eavEntityAttribute
     * @param Collection $collection
     * @param array $components
     * @param array $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        AttributeRepository $attributeRepository,

        Attribute $eavEntityAttribute,
        Collection $collection,
        array $components = [],
        array $data = []
    ) {
        parent::__construct($context, $uiComponentFactory, $components, $data);
        $this->_eavEntityAttribute = $eavEntityAttribute;
        $this->collection = $collection;
        $this->attributeRepository = $attributeRepository;
    }

    /**
     * @param array $dataSource
     * @return array
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            $attributes = $this->attributeRepository->get('color');
            $fieldName = $this->getData('name');
            $product = $this->collection->load()->addAttributeToSelect('color');

            foreach ($dataSource['data']['items'] as & $item) {
                $attrStrings = [];
                foreach ($attributes as $attributeCode => $attribute) {
                    $item['color'] = $product->addAttributeToFilter('color');
                    if (isset($item[$attributeCode])) {
                        $attrStrings[] = $attribute['color'] . ': ' . $attribute['options'][$item[$attributeCode]];
                    }
                    $item[$fieldName] = implode(', ', $attrStrings);
                }
            }
        }

        return $dataSource;
    }

    public function toOptionArray()
    {
        return $this->_eavEntityAttribute->getAttributeCodesByFrontendType('color');
    }
}
